@@ -574,9 +574,17 @@ where
         punct!(self, "@");
         keyword!(self, "page");
 
-        if n.prelude.is_some() {
-            space!(self);
-            emit!(self, n.prelude);
+        if let Some(prelude) = &n.prelude {
+            match prelude.selectors.get(0) {
+                Some(page_selector) if page_selector.page_type.is_none() => {
+                    formatting_space!(self);
+                }
+                _ => {
+                    space!(self);
+                }
+            }
+
+            emit!(self, prelude);
         } else {
             formatting_space!(self);
         }
@@ -638,7 +646,7 @@ where
     fn emit_page_margin_rule(&mut self, n: &PageMarginRule) -> Result {
         punct!(self, "@");
         emit!(self, n.name);
-        space!(self);
+        formatting_space!(self);
         punct!(self, "{");
         self.emit_list(&n.block, ListFormat::SemiDelimited | ListFormat::MultiLine)?;
         punct!(self, "}");
@@ -698,7 +706,7 @@ where
         keyword!(self, "document");
         space!(self);
 
-        self.emit_list(&n.selectors, ListFormat::CommaDelimited)?;
+        self.emit_list(&n.matching_functions, ListFormat::CommaDelimited)?;
 
         formatting_space!(self);
 
@@ -707,13 +715,22 @@ where
         punct!(self, "}");
     }
 
+    #[emitter]
+    fn emit_document_rule_matching_function(&mut self, n: &DocumentRuleMatchingFunction) -> Result {
+        match n {
+            DocumentRuleMatchingFunction::Url(n) => emit!(self, n),
+            DocumentRuleMatchingFunction::Function(n) => emit!(self, n),
+        }
+    }
+
     fn emit_list_values(&mut self, nodes: &[Value], format: ListFormat) -> Result {
         let iter = nodes.iter();
+        let len = nodes.len();
 
         for (idx, node) in iter.enumerate() {
             emit!(self, node);
 
-            if idx != nodes.len() - 1 {
+            if idx != len - 1 {
                 let need_delim = match node {
                     Value::SimpleBlock(_)
                     | Value::Function(_)
@@ -727,6 +744,31 @@ where
                         })) => false,
                         _ => !self.config.minify,
                     },
+                    Value::Ident(_) => match nodes.get(idx + 1) {
+                        Some(Value::SimpleBlock(_))
+                        | Some(Value::Color(Color::HexColor(_)))
+                        | Some(Value::Str(_)) => !self.config.minify,
+                        Some(Value::Delimiter(_)) => false,
+                        Some(Value::Number(n)) => {
+                            if self.config.minify {
+                                let minified = minify_numeric(n.value);
+
+                                !minified.starts_with('.')
+                            } else {
+                                true
+                            }
+                        }
+                        Some(Value::Dimension(n)) => {
+                            if self.config.minify {
+                                let minified = minify_numeric(n.value.value);
+
+                                !minified.starts_with('.')
+                            } else {
+                                true
+                            }
+                        }
+                        _ => true,
+                    },
                     _ => match nodes.get(idx + 1) {
                         Some(Value::SimpleBlock(_)) | Some(Value::Color(Color::HexColor(_))) => {
                             !self.config.minify
@@ -968,52 +1010,9 @@ where
     #[emitter]
     fn emit_number(&mut self, n: &Number) -> Result {
         if self.config.minify {
-            if n.value.is_sign_negative() && n.value == 0.0 {
-                self.wr.write_raw(Some(n.span), "-0")?;
-            } else {
-                let mut minified = n.value.to_string();
+            let minified = minify_numeric(n.value);
 
-                if minified.starts_with("0.") {
-                    minified.replace_range(0..1, "");
-                } else if minified.starts_with("-0.") {
-                    minified.replace_range(1..2, "");
-                }
-
-                if minified.starts_with(".000") {
-                    let mut cnt = 3;
-
-                    for &v in minified.as_bytes().iter().skip(4) {
-                        if v == b'0' {
-                            cnt += 1;
-                        } else {
-                            break;
-                        }
-                    }
-
-                    minified.replace_range(0..cnt + 1, "");
-
-                    let remain_len = minified.len();
-
-                    minified.push_str("e-");
-                    minified.push_str(&(remain_len + cnt).to_string());
-                } else if minified.ends_with("000") {
-                    let mut cnt = 3;
-
-                    for &v in minified.as_bytes().iter().rev().skip(3) {
-                        if v == b'0' {
-                            cnt += 1;
-                        } else {
-                            break;
-                        }
-                    }
-
-                    minified.truncate(minified.len() - cnt);
-                    minified.push('e');
-                    minified.push_str(&cnt.to_string());
-                }
-
-                self.wr.write_raw(Some(n.span), &minified)?;
-            }
+            self.wr.write_raw(Some(n.span), &minified)?;
         } else {
             self.wr.write_raw(Some(n.span), &n.raw)?;
         }
@@ -1502,6 +1501,54 @@ where
     }
 }
 
+fn minify_numeric(value: f64) -> String {
+    if value.is_sign_negative() && value == 0.0 {
+        return "-0".to_owned();
+    }
+    let mut minified = value.to_string();
+
+    if minified.starts_with("0.") {
+        minified.replace_range(0..1, "");
+    } else if minified.starts_with("-0.") {
+        minified.replace_range(1..2, "");
+    }
+
+    if minified.starts_with(".000") {
+        let mut cnt = 3;
+
+        for &v in minified.as_bytes().iter().skip(4) {
+            if v == b'0' {
+                cnt += 1;
+            } else {
+                break;
+            }
+        }
+
+        minified.replace_range(0..cnt + 1, "");
+
+        let remain_len = minified.len();
+
+        minified.push_str("e-");
+        minified.push_str(&(remain_len + cnt).to_string());
+    } else if minified.ends_with("000") {
+        let mut cnt = 3;
+
+        for &v in minified.as_bytes().iter().rev().skip(3) {
+            if v == b'0' {
+                cnt += 1;
+            } else {
+                break;
+            }
+        }
+
+        minified.truncate(minified.len() - cnt);
+        minified.push('e');
+        minified.push_str(&cnt.to_string());
+    }
+
+    minified
+}
+
 fn minify_hex_color(value: &str) -> String {
     let length = value.len();
 
